import * as _floating_ui_core_src_types from '@floating-ui/core/src/types';
import * as vue_demi from 'vue-demi';
import { PropType, UnwrapRef, ExtractPropTypes } from 'vue-demi';
import { R as ReferenceType, d as UseAutoUpdateOptions, e as UseFloatingData, F as FloatingType } from '../useAutoUpdate-b2e62d09.js';
import { Placement, Strategy, Middleware } from '@floating-ui/core';
import '@floating-ui/dom';

declare const FloatingCreator: vue_demi.DefineComponent<{
    readonly disabled: BooleanConstructor;
    readonly reference: {
        type: vue_demi.PropType<ReferenceType | null>;
    };
    readonly floating: {
        type: vue_demi.PropType<HTMLElement | null>;
    };
    readonly placement: {
        readonly type: vue_demi.PropType<_floating_ui_core_src_types.Placement>;
        readonly default: _floating_ui_core_src_types.Placement;
    };
    readonly strategy: {
        readonly type: vue_demi.PropType<_floating_ui_core_src_types.Strategy>;
        readonly default: "absolute";
    };
    readonly middleware: vue_demi.PropType<_floating_ui_core_src_types.Middleware[]>;
    readonly autoUpdate: {
        readonly type: vue_demi.PropType<boolean | UseAutoUpdateOptions>;
        readonly default: true;
    };
    readonly onUpdate: vue_demi.PropType<(data: {
        x: number | null;
        y: number | null;
        strategy: _floating_ui_core_src_types.Strategy;
        placement: _floating_ui_core_src_types.Placement;
        middlewareData: {
            [x: string]: any;
            arrow?: {
                x?: number | undefined;
                y?: number | undefined;
                centerOffset: number;
            } | undefined;
            autoPlacement?: {
                index?: number | undefined;
                overflows: {
                    placement: _floating_ui_core_src_types.Placement;
                    overflows: number[];
                }[];
            } | undefined;
            flip?: {
                index?: number | undefined;
                overflows: {
                    placement: _floating_ui_core_src_types.Placement;
                    overflows: number[];
                }[];
            } | undefined;
            hide?: {
                referenceHidden?: boolean | undefined;
                escaped?: boolean | undefined;
                referenceHiddenOffsets?: {
                    top: number;
                    right: number;
                    bottom: number;
                    left: number;
                } | undefined;
                escapedOffsets?: {
                    top: number;
                    right: number;
                    bottom: number;
                    left: number;
                } | undefined;
            } | undefined;
            offset?: {
                x: number;
                y: number;
            } | undefined;
            shift?: {
                x: number;
                y: number;
            } | undefined;
        };
    }) => void>;
}, () => vue_demi.VNode<vue_demi.RendererNode, vue_demi.RendererElement, {
    [key: string]: any;
}>[] | undefined, unknown, {}, {}, vue_demi.ComponentOptionsMixin, vue_demi.ComponentOptionsMixin, {}, string, vue_demi.VNodeProps & vue_demi.AllowedComponentProps & vue_demi.ComponentCustomProps, Readonly<vue_demi.ExtractPropTypes<{
    readonly disabled: BooleanConstructor;
    readonly reference: {
        type: vue_demi.PropType<ReferenceType | null>;
    };
    readonly floating: {
        type: vue_demi.PropType<HTMLElement | null>;
    };
    readonly placement: {
        readonly type: vue_demi.PropType<_floating_ui_core_src_types.Placement>;
        readonly default: _floating_ui_core_src_types.Placement;
    };
    readonly strategy: {
        readonly type: vue_demi.PropType<_floating_ui_core_src_types.Strategy>;
        readonly default: "absolute";
    };
    readonly middleware: vue_demi.PropType<_floating_ui_core_src_types.Middleware[]>;
    readonly autoUpdate: {
        readonly type: vue_demi.PropType<boolean | UseAutoUpdateOptions>;
        readonly default: true;
    };
    readonly onUpdate: vue_demi.PropType<(data: {
        x: number | null;
        y: number | null;
        strategy: _floating_ui_core_src_types.Strategy;
        placement: _floating_ui_core_src_types.Placement;
        middlewareData: {
            [x: string]: any;
            arrow?: {
                x?: number | undefined;
                y?: number | undefined;
                centerOffset: number;
            } | undefined;
            autoPlacement?: {
                index?: number | undefined;
                overflows: {
                    placement: _floating_ui_core_src_types.Placement;
                    overflows: number[];
                }[];
            } | undefined;
            flip?: {
                index?: number | undefined;
                overflows: {
                    placement: _floating_ui_core_src_types.Placement;
                    overflows: number[];
                }[];
            } | undefined;
            hide?: {
                referenceHidden?: boolean | undefined;
                escaped?: boolean | undefined;
                referenceHiddenOffsets?: {
                    top: number;
                    right: number;
                    bottom: number;
                    left: number;
                } | undefined;
                escapedOffsets?: {
                    top: number;
                    right: number;
                    bottom: number;
                    left: number;
                } | undefined;
            } | undefined;
            offset?: {
                x: number;
                y: number;
            } | undefined;
            shift?: {
                x: number;
                y: number;
            } | undefined;
        };
    }) => void>;
}>>, {
    readonly placement: _floating_ui_core_src_types.Placement;
    readonly strategy: _floating_ui_core_src_types.Strategy;
    readonly disabled: boolean;
    readonly autoUpdate: boolean | UseAutoUpdateOptions;
}>;

declare type FloatingData = UseFloatingData;
declare type AutoUpdateOptions = UseAutoUpdateOptions;
declare type FloatingCreatorSlotProps = FloatingData;
interface FloatingCreatorExposed {
    getFloatingData: () => FloatingData;
    updatePosition: () => void;
}
declare const FloatingCreatorProps: {
    /**
     * Whether to disable the floating.
     */
    readonly disabled: BooleanConstructor;
    /**
     * Reference element.
     *
     * You can position a floating element relative to a virtual element instead of a real one.
     * This enables things like positioning context menus or following the cursor.
     * @see https://floating-ui.com/docs/virtual-elements
     */
    readonly reference: {
        type: PropType<ReferenceType | null>;
    };
    /**
     * Floating element.
     */
    readonly floating: {
        type: PropType<FloatingType | null>;
    };
    /**
     * Where to place the floating element relative to its reference element.
     * @see https://floating-ui.com/docs/computePosition#placement
     */
    readonly placement: {
        readonly type: PropType<Placement>;
        readonly default: Placement;
    };
    /**
     * This is the type of CSS position property to use.
     * @see https://floating-ui.com/docs/computePosition#strategy
     */
    readonly strategy: {
        readonly type: PropType<Strategy>;
        readonly default: "absolute";
    };
    /**
     * They alter the positioning coordinates from the basic placement to customize the position,
     * provide data, or act as "visibility optimization" techniques.
     * @see https://floating-ui.com/docs/computePosition#middleware
     */
    readonly middleware: PropType<Middleware[]>;
    /**
     * This function adds listeners that automatically call an update function when necessary
     * so that the floating element remains “anchored” to the reference element in a variety of scenarios without detaching.
     */
    readonly autoUpdate: {
        readonly type: PropType<boolean | UseAutoUpdateOptions>;
        readonly default: true;
    };
    /**
     * Callback on floating data status changes.
     */
    readonly onUpdate: PropType<(data: UnwrapRef<FloatingData>) => void>;
};
declare type FloatingCreatorProps = ExtractPropTypes<typeof FloatingCreatorProps>;

export { AutoUpdateOptions, FloatingCreator, FloatingCreatorExposed, FloatingCreatorProps, FloatingCreatorSlotProps, FloatingData };
