import { VirtualElement, AutoUpdateOptions } from '@floating-ui/dom';
import { Ref } from 'vue-demi';
import { Strategy, Placement, MiddlewareData, ComputePositionConfig } from '@floating-ui/core';

declare type ReferenceType = Element | VirtualElement;
declare type FloatingType = HTMLElement;
declare type MaybeRef<T> = T | Ref<T>;
declare type MaybeRefType<T> = MaybeRef<T | undefined | null>;
declare type MaybeReferenceRef = MaybeRefType<ReferenceType>;
declare type MaybeFloatingRef = MaybeRefType<FloatingType>;
interface UseFloatingData {
    x: number | null;
    y: number | null;
    strategy: Strategy;
    placement: Placement;
    middlewareData: MiddlewareData;
}
declare type UseFloatingOptions = Omit<ComputePositionConfig, 'platform'> & {
    disabled?: boolean;
};
interface UseFloatingReturn {
    /**
     * @see https://floating-ui.com/docs/computePosition#return-value
     */
    data: Readonly<Ref<UseFloatingData>>;
    update: () => void;
    stop: () => void;
}

declare type UseAutoUpdateOptions = Partial<AutoUpdateOptions> & {
    /**
     * @default false
     */
    disabled?: boolean;
};
/**
 * Automatically updates the position of the floating element when necessary.
 * @see https://floating-ui.com/docs/autoUpdate
 */
declare function useAutoUpdate(reference: MaybeReferenceRef, floating: MaybeFloatingRef, update: () => void, options?: MaybeRef<UseAutoUpdateOptions>): () => void;

export { FloatingType as F, MaybeReferenceRef as M, ReferenceType as R, UseFloatingOptions as U, MaybeFloatingRef as a, MaybeRef as b, UseFloatingReturn as c, UseAutoUpdateOptions as d, UseFloatingData as e, useAutoUpdate as u };
